import requests
import tkinter as tk
import json
from datetime import datetime
from geopy.geocoders import Nominatim
from tkinter import ttk


################################# 
### SETTING UP THE GEOLOCATER ###
################################# 

geolocater = Nominatim(user_agent='Weather_App')

#################
### FUNCTIONS ###
#################

def getCoordinate(location):
    try:
        c = geolocater.geocode(location)
        return c
    except Exception:
        return "Location Server not found"

def getWeather(c):
    if c == None:
        return "Location not found"
    elif c == "Location Server not found":
        return "Location Server not found"
    lon = c.longitude
    lat = c.latitude
    url = 'http://www.7timer.info/bin/api.pl?lon=' + str(lon) + '&lat=' + str(lat) + '&product=civillight&output=json'
    response = requests.get(url)
    if response.status_code == 200: # if OK        
        try:
            w_data = response.json()            
            return w_data        
        except Exception:            
            return "Data not available"    
    else:        
        return "Server Error"

def displayWeatherData(e=None):
    loc = cmb_locations.get()
    c = getCoordinate(loc)
    w_data = getWeather(c)
    if w_data == 'Weather Server Error':
        errorMessage.set("Could not reach weather server")
        lbl_status.grid(row=1, column=0,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
    elif w_data == "Data not available":
        errorMessage.set("Data not available")
        lbl_status.grid(row=1, column=0,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
    elif w_data == "Location not found":            
        errorMessage.set("404 Error: Location could not be found")
        lbl_status.grid(row=1, column=0,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)        
    elif w_data == "Location Server not found":            
        errorMessage.set("404 Error: Location Server not available")
        lbl_status.grid(row=1, column=0,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
    else:
        lbl_location_name['text'] = '{} ({})'.format(loc,c)
        displayToday(w_data)
        display7Days(w_data)

def displayToday(w_data):
    today = w_data['dataseries'][0]
    dd = datetime.strptime(str(today['date']), '%Y%m%d')
    dd_display = dd.strftime('%A, %-d %B %Y')
    weather = today['weather']
    min_temp = today['temp2m']['min']
    max_temp = today['temp2m']['max']
    windspeed = today['wind10m_max']
    
    #Display data on widget
    lbl_date['text'] = dd_display
    lbl_maxtemp['text'] = "Max Temp: {}℃".format(max_temp)
    lbl_mintemp['text'] = "Min Temp: {}℃".format(min_temp)
    displayWeatherCondition(weather)
    displayWindCondtion(windspeed)

def display7Days(w_data):
    for i in range(7):
        data = w_data['dataseries'][i]
        dd = datetime.strptime(str(data['date']), '%Y%m%d')
        dd_display = dd.strftime('%A, %-d %B %Y')
        weather = data['weather']
        min_temp = data['temp2m']['min']
        max_temp = data['temp2m']['max']
        
        #Display data on widget
        seven_days_dates[i]['text'] = dd_display
        seven_days_weather[i]['text'] = displayWeatherCondition(weather, textOnly = True)
        seven_days_temperatures[i]['text'] = "{}℃ —— {}℃".format(min_temp, max_temp)
        





weatherConditions = {
    'clear': 'Clear',
    'humid': 'Humid',
    'lightrain': 'Light Rain',
    'oshower': 'Light Rain',
    'rain': 'Rain',
    'pcloudy': 'Partly Cloudy',
    'cloudy': 'Cloudy',
    'rainsnow': 'Rain and Snow',
    'lightsnow': 'Light Snow',
    'snow': 'Snow',
    'mcloudy': 'Medium Cloudy'
}

def displayWeatherCondition(condition, textOnly = False):
    if textOnly:
        return weatherConditions[condition] if condition in weatherConditions else condition
    else:
        if condition in weatherConditions:
            lbl_condition['text'] = weatherConditions[condition]
        else:
            lbl_condition['text'] = condition
        if condition in weatherConditionsImages:
            img = weatherConditionsImages[condition]
        else:
            img = img_clear
        canvas_weather.delete('all')
        left = (120 - img.width())/2+8
        top = (120 - img.height())/2+8
        canvas_weather.create_image(left, top, anchor=tk.NW, image=img)               

windConditions = {
    1: 'Calm (Below 0.3m/s',
    2: 'Light (0.3-3.4m/s)',
    3: 'Moderate (3.4-8m/s)',
    4: 'Fresh (8-10.8m/s)',
    5: 'Strong (10.8-17.2m/s)',
    6: 'Gale (17.2-24.5m/s)',
    7: 'Storm (24.5-32.6m/s)',
    8: 'Hurricane (Over 32.6m/s)'
}



def displayWindCondtion(speed):
    windCondition.set(windConditions[speed])

###################
### WEATHER GUI ###
###################

window = tk.Tk()
window.geometry('500x560+200+200')
window.title('Weather Application')
window.resizable(False,False)
window.columnconfigure(0, weight=1)
window.rowconfigure(2, weight = 1)


################# 
### Variables ###
#################
locations = ['Singapore', 'Tokyo, Japan', 'New York, USA', 'London', 'Sydney, Australia', 'Columbia', 'Shanghai, China', 'Hong Kong, China', 'Auckland, New Zealand', 'Washington D.C, USA', 'Michigan, USA', 'Rome, Italy',
    'Queenstown, New Zealand', 'Honolulu, Hawaii']
errorMessage = tk.StringVar()
currentView = 'today'


frm_input = tk.Frame(master=window, bg='blue')
frm_input.grid(row=0,column=0,sticky=tk.EW)
frm_input.columnconfigure(1, weight=1)

lbl_location = tk.Label(
    master = frm_input,
    text = 'Location: ',
    bg = 'blue',
    fg = 'white',
    font = ('Helvetica Neue', 18)
)

cmb_locations = ttk.Combobox(
    master=frm_input, 
    values = locations
)

cmb_locations.bind('<Return>', displayWeatherData)

btn_confirm = tk.Button(
    master = frm_input,
    text = '⛅️',
    bg = 'blue',
    font = ('Helvetica Neue', 12),
    command = displayWeatherData
)

lbl_location.grid(row=0,column=0,sticky='nsew', padx=4, pady=4)
cmb_locations.grid(row=0,column=1,sticky='nsew', padx=4, pady=4)
btn_confirm.grid(row=0,column=2,sticky='nsew', padx=4, pady=4, ipadx=5)

lbl_status = tk.Label(
    master = window,
    textvariable = errorMessage,
    foreground = 'red',
    font = ('Helvetica Neue', 20, 'bold')
)


####################
### OUTPUT FRAME ###
####################

frm_output = tk.Frame(master=window,bg='grey50')
frm_output.grid(row=2, column=0, sticky=tk.NSEW)
frm_output.columnconfigure(0, weight=1)
frm_output.rowconfigure(4, weight=1)

lbl_location_name = tk.Label(
    master=frm_output,
    text = 'Location: N/A ',
    font = ('Helvetica Neue', 18, "bold"),
    anchor = tk.W,
    justify = tk.LEFT,
    padx=10,
    wraplength = 450
    )
lbl_location_name.grid(row=0, column=0, columnspan=2,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)

frm_output_today = tk.Frame(frm_output, background = 'light green')
frm_output_today.grid(row=1, column=0, sticky=tk.NSEW, padx=5, pady=5, ipadx=5, ipady=5)
frm_output_today.columnconfigure([0,1], weight=1)

lbl_date = tk.Label(
    master=frm_output_today,
    text = "Today: N/A",
    font = ('Helvetica Neue', 18),
    padx = 10,
    justify = tk.LEFT,
    anchor = tk.W
)

lbl_condition = tk.Label(
    master=frm_output_today,
    text = "Weather Condition:",
    font = ('Helvetica Neue', 18),
    padx = 10
)

lbl_temperature = tk.Label(
    master=frm_output_today,
    text = "Temperature",
    font = ('Helvetica Neue', 18),
    padx = 10,
    justify = tk.LEFT,
    anchor = tk.W
)
lbl_maxtemp = tk.Label(
    master=frm_output_today,
    text = "Max Temp: ",
    font = ('Helvetica Neue', 14),
    padx = 10,
    justify = tk.LEFT,
    anchor = tk.W
)
lbl_mintemp = tk.Label(
    master=frm_output_today,
    text = "Min Temp: ",
    font = ('Helvetica Neue', 14),
    padx = 10,
    justify = tk.LEFT,
    anchor = tk.W
)

canvas_weather = tk.Canvas(
    master=frm_output_today,
    width = 120,
    height=  120,
    borderwidth = 1,
    relief = tk.SOLID
)

lbl_date.grid(row=0, column=0, columnspan=2,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
lbl_condition.grid(row=1, column=0,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
lbl_temperature.grid(row=1, column=1,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
lbl_maxtemp.grid(row=2, column=1,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
lbl_mintemp.grid(row=3, column=1,sticky=tk.EW, padx=5,pady=5, ipadx=5, ipady = 5)
canvas_weather.grid(row=2, column=0,rowspan=2)



############
## IMAGES ##
############

img_clear = tk.PhotoImage(file='images/clear.png')
img_clear = img_clear.zoom(2)
img_cloudy = tk.PhotoImage(file='images/cloudy.png')
img_cloudy = img_cloudy.zoom(2)
img_pcloudy = tk.PhotoImage(file='images/pcloudy.png')
img_pcloudy = img_pcloudy.zoom(2)
img_rain = tk.PhotoImage(file='images/rain.png')
img_rain = img_rain.zoom(2)
img_snow = tk.PhotoImage(file='images/snow.png')
img_snow = img_snow.zoom(2)
img_thunderstorm = tk.PhotoImage(file='images/thunderstorm.png')
img_thunderstorm = img_thunderstorm.zoom(2)
img_rainthunderstorm = tk.PhotoImage(file='images/rainthunderstorm.png')
img_rainthunderstorm = img_rainthunderstorm.zoom(2)

weatherConditionsImages = {
    'clear': img_clear,
    'humid': img_clear,
    'lightrain': img_rain,
    'oshower': img_rain,
    'rain': img_rain,
    'pcloudy': img_pcloudy,
    'cloudy': img_cloudy,
    'mcloudy': img_pcloudy,
    'rainsnow': img_snow,
    'lightsnow': img_snow,
    'snow': img_snow
}


frm_wind = tk.Frame(frm_output_today)
frm_wind.grid(row=5,column=0,columnspan=2, sticky=tk.EW, padx=5, pady=5, ipady=5, ipadx=5)

lbl_wind = tk.Label(
    master = frm_wind,
    text = 'Wind: ',
    font = ('Helvetica Neue', 18),
    anchor = tk.W,
    padx = 10
)
lbl_wind.grid(row=0, column=0, sticky=tk.EW)

windCondition = tk.StringVar()

lbl_wind_condition = tk.Label(
    master = frm_wind,
    textvariable = windCondition,
    font = ('Helvetica Neue', 16),
    anchor = tk.W
)

lbl_wind_condition.grid(row=0, column=1, sticky=tk.EW, padx=10)

frm_output_7days = tk.Frame(frm_output, background='light blue')
frm_output_7days.columnconfigure([0,1,2], weight = 1)

seven_days_dates = []
seven_days_weather = []
seven_days_temperatures = []

for i in range(7):
    seven_days_dates.append(tk.Label(frm_output_7days, background='yellow', text='day' + str(i+1)))
    seven_days_dates[i].grid(row=i, column=0, sticky=tk.EW, padx=5, pady=5, ipadx=5, ipady=5)
    seven_days_weather.append(tk.Label(frm_output_7days, background='pink', text='weather' + str(i+1)))
    seven_days_weather[i].grid(row=i, column=1, sticky=tk.EW, padx=5, pady=5, ipadx=5, ipady=5)
    seven_days_temperatures.append(tk.Label(frm_output_7days, background='lime green', text='temperature' + str(i+1)))
    seven_days_temperatures[i].grid(row=i, column=2, sticky=tk.EW, padx=5, pady=5, ipadx=5, ipady=5)

####################
### VIEW SWITCHER ##
####################

frm_buttons = tk.Frame(frm_output)
frm_buttons.grid(row=3, column=0, sticky=tk.EW, padx=5, pady=5, ipadx=5, ipady=5)
frm_buttons.columnconfigure([0,1], weight=1)

def switchview():
    global currentView
    if currentView == '7day':
        currentView = 'today'
        frm_output_7days.grid_remove()
        frm_output_today.grid(row=1, column=0, sticky=tk.NSEW, padx=5, pady=5, ipadx=5, ipady=5)
        btn_7days['state'] = 'normal'
        btn_today['state'] = 'disabled'
    else:
        currentView = '7day'
        frm_output_today.grid_remove()
        frm_output_7days.grid(row=1,column=0,sticky=tk.NSEW,padx=5,pady=5,ipadx=5,ipady=5)
        btn_7days['state'] = 'disabled'
        btn_today['state'] = 'normal'

btn_today = tk.Button(
    master = frm_buttons,
    text = 'Today',
    state = 'disabled',
    font = ('Helvetica Neue', 18, 'bold'),
    command = switchview 
)
btn_today.grid(row=0, column=0, sticky=tk.EW, padx=5, pady=5, ipadx=5, ipady=5)

btn_7days = tk.Button(
    master = frm_buttons,
    text = '7 days',
    font = ('Helvetica Neue', 18, 'bold'),
    command = switchview
)
btn_7days.grid(row=0, column=1, sticky=tk.EW, padx=5, pady=5, ipadx=5, ipady=5)

frm_bottom = tk.Frame(frm_output)
frm_bottom.grid(row=4, column=0, pady=5, sticky = tk.S)

lbl_bottom = tk.Label(
    master = frm_bottom,
    text = 'Desigined and scripted by Asher Mendelsohn © 2021',
    font = ('Helvetica Neue', 14)
)

lbl_bottom.grid(row=0, column=0, sticky=tk.S, padx=5, pady=5, ipadx = 2)


window.mainloop()
